@@ -37,13 +37,13 @@ class fMRIPlot(object):
         func_nii = nb.load(func_file)
         self.tr = tr if tr is not None else func_nii.header.get_zooms()[-1]
 
-        self.mask_data = np.ones_like(func_nii.get_data(), dtype='uint8')
+        self.mask_data = nb.fileslice.strided_scalar(func_nii.shape[:3], np.uint8(1))
         if mask_file:
-            self.mask_data = nb.load(mask_file).get_data().astype('uint8')
+            self.mask_data = np.asanyarray(nb.load(mask_file).dataobj).astype('uint8')
 
         self.seg_data = None
         if seg_file:
-            self.seg_data = nb.load(seg_file).get_data()
+            self.seg_data = np.asanyarray(nb.load(seg_file).dataobj)
 
         if units is None:
             units = {}
@@ -101,14 +101,14 @@ class fMRIPlot(object):
                 name=name, **kwargs)
             grid_id += 1
 
-        plot_carpet(self.func_file, self.seg_data, subplot=grid[-1])
+        plot_carpet(self.func_file, self.seg_data, subplot=grid[-1], tr=self.tr)
         # spikesplot_cb([0.7, 0.78, 0.2, 0.008])
         return figure
 
 
 def plot_carpet(img, atlaslabels, detrend=True, nskip=0, size=(950, 800),
                 subplot=None, title=None, output_file=None, legend=False,
-                lut=None):
+                lut=None, tr=None):
     """
     Plot an image representation of voxel intensities across time also know
     as the "carpet plot" or "Power plot". See Jonathan Power Neuroimage
@@ -141,12 +141,19 @@ def plot_carpet(img, atlaslabels, detrend=True, nskip=0, size=(950, 800),
         legend : bool
             Whether to render the average functional series with ``atlaslabels`` as
             overlay.
+        tr : float , optional
+            Specify the TR, if specified it uses this value. If left as None,
+            # Frames is plotted instead of time.
     """
-    img_nii = check_niimg_4d(img, dtype='auto')
-    func_data = _safe_get_data(img_nii, ensure_finite=True)
 
     # Define TR and number of frames
-    tr = img_nii.header.get_zooms()[-1]
+    notr = False
+    if tr is None:
+        notr = True
+        tr = 1.
+
+    img_nii = check_niimg_4d(img, dtype='auto',)
+    func_data = _safe_get_data(img_nii, ensure_finite=True)
     ntsteps = func_data.shape[-1]
 
     data = func_data[atlaslabels > 0].reshape(-1, ntsteps)
@@ -217,7 +224,10 @@ def plot_carpet(img, atlaslabels, detrend=True, nskip=0, size=(950, 800),
     interval = max((int(data.shape[-1] + 1) // 10, int(data.shape[-1] + 1) // 5, 1))
     xticks = list(range(0, data.shape[-1])[::interval])
     ax1.set_xticks(xticks)
-    ax1.set_xlabel('time (s)')
+    if notr:
+        ax1.set_xlabel('time (frame #)')
+    else:
+        ax1.set_xlabel('time (s)')
     labels = tr * (np.array(xticks)) * t_dec
     ax1.set_xticklabels(['%.02f' % t for t in labels.tolist()], fontsize=5)
 
@@ -463,30 +473,35 @@ def confoundplot(tseries, gs_ts, gs_dist=None, name=None,
     ax_ts.spines["left"].set_visible(False)
     # ax_ts.yaxis.set_ticks_position('left')
 
-    # Calculate Y limits
-    def_ylims = [tseries[~np.isnan(tseries)].min() - 0.1 * abs(tseries[~np.isnan(tseries)].min()),
-                 1.1 * tseries[~np.isnan(tseries)].max()]
-    if ylims is not None:
-        if ylims[0] is not None:
-            def_ylims[0] = min([def_ylims[0], ylims[0]])
-        if ylims[1] is not None:
-            def_ylims[1] = max([def_ylims[1], ylims[1]])
-
-    # Add space for plot title and mean/SD annotation
-    def_ylims[0] -= 0.1 * (def_ylims[1] - def_ylims[0])
-
-    ax_ts.set_ylim(def_ylims)
-    # yticks = sorted(def_ylims)
     ax_ts.set_yticks([])
     ax_ts.set_yticklabels([])
-    # ax_ts.set_yticks(yticks)
-    # ax_ts.set_yticklabels(['%.02f' % y for y in yticks])
 
-    # Annotate stats
-    maxv = tseries[~np.isnan(tseries)].max()
-    mean = tseries[~np.isnan(tseries)].mean()
-    stdv = tseries[~np.isnan(tseries)].std()
-    p95 = np.percentile(tseries[~np.isnan(tseries)], 95.0)
+    nonnan = tseries[~np.isnan(tseries)]
+    if nonnan.size > 0:
+        # Calculate Y limits
+        valrange = (nonnan.max() - nonnan.min())
+        def_ylims = [nonnan.min() - 0.1 * valrange, nonnan.max() + 0.1 * valrange]
+        if ylims is not None:
+            if ylims[0] is not None:
+                def_ylims[0] = min([def_ylims[0], ylims[0]])
+            if ylims[1] is not None:
+                def_ylims[1] = max([def_ylims[1], ylims[1]])
+
+        # Add space for plot title and mean/SD annotation
+        def_ylims[0] -= 0.1 * (def_ylims[1] - def_ylims[0])
+
+        ax_ts.set_ylim(def_ylims)
+
+        # Annotate stats
+        maxv = nonnan.max()
+        mean = nonnan.mean()
+        stdv = nonnan.std()
+        p95 = np.percentile(nonnan, 95.0)
+    else:
+        maxv = 0
+        mean = 0
+        stdv = 0
+        p95 = 0
 
     stats_label = (r'max: {max:.3f}{units} $\bullet$ mean: {mean:.3f}{units} '
                    r'$\bullet$ $\sigma$: {sigma:.3f}').format(
@@ -530,3 +545,228 @@ def confoundplot(tseries, gs_ts, gs_dist=None, name=None,
 
         return [ax_ts, ax_dist], gs
     return ax_ts, gs
+
+
+def compcor_variance_plot(metadata_files, metadata_sources=None,
+                          output_file=None, varexp_thresh=(0.5, 0.7, 0.9),
+                          fig=None):
+    """
+    Parameters
+    ----------
+    metadata_files: list
+        List of paths to files containing component metadata. If more than one
+        decomposition has been performed (e.g., anatomical and temporal
+        CompCor decompositions), then all metadata files can be provided in
+        the list. However, each metadata file should have a corresponding
+        entry in `metadata_sources`.
+    metadata_sources: list or None
+        List of source names (e.g., ['aCompCor']) for decompositions. This
+        list should be of the same length as `metadata_files`.
+    output_file: str or None
+        Path where the output figure should be saved. If this is not defined,
+        then the plotting axes will be returned instead of the saved figure
+        path.
+    varexp_thresh: tuple
+        Set of variance thresholds to include in the plot (default 0.5, 0.7,
+        0.9).
+    fig: figure or None
+        Existing figure on which to plot.
+
+    Returns
+    -------
+    ax: axes
+        Plotting axes. Returned only if the `output_file` parameter is None.
+    output_file: str
+        The file where the figure is saved.
+    """
+    metadata = {}
+    if metadata_sources is None:
+        if len(metadata_files) == 1:
+            metadata_sources = ['CompCor']
+        else:
+            metadata_sources = ['Decomposition {:d}'.format(i)
+                                for i in range(len(metadata_files))]
+    for file, source in zip(metadata_files, metadata_sources):
+        metadata[source] = pd.read_csv(str(file), sep=r'\s+')
+        metadata[source]['source'] = source
+    metadata = pd.concat(list(metadata.values()))
+    bbox_txt = {
+        'boxstyle': 'round',
+        'fc': 'white',
+        'ec': 'none',
+        'color': 'none',
+        'linewidth': 0,
+        'alpha': 0.8
+    }
+
+    decompositions = []
+    data_sources = list(metadata.groupby(['source', 'mask']).groups.keys())
+    for source, mask in data_sources:
+        if not np.isnan(
+                metadata.loc[
+                    (metadata['source'] == source) & (metadata['mask'] == mask)
+                ]['singular_value'].values[0]):
+            decompositions.append((source, mask))
+
+    if fig is not None:
+        ax = [fig.add_subplot(1, len(decompositions), i + 1)
+              for i in range(len(decompositions))]
+    elif len(decompositions) > 1:
+        fig, ax = plt.subplots(1, len(decompositions),
+                               figsize=(5 * len(decompositions), 5))
+    else:
+        ax = [plt.axes()]
+
+    for m, (source, mask) in enumerate(decompositions):
+        components = metadata[(metadata['mask'] == mask) & (metadata['source'] == source)]
+        if len([m for s, m in decompositions if s == source]) > 1:
+            title_mask = ' ({} mask)'.format(mask)
+        else:
+            title_mask = ''
+        fig_title = '{}{}'.format(source, title_mask)
+
+        ax[m].plot(np.arange(components.shape[0] + 1), [0] + list(
+            100 * components['cumulative_variance_explained']),
+            color='purple',
+            linewidth=2.5)
+        ax[m].grid(False)
+        ax[m].set_xlabel('number of components in model')
+        ax[m].set_ylabel('cumulative variance explained (%)')
+        ax[m].set_title(fig_title)
+
+        varexp = {}
+
+        for i, thr in enumerate(varexp_thresh):
+            varexp[thr] = np.atleast_1d(np.searchsorted(
+                components['cumulative_variance_explained'], thr)) + 1
+            ax[m].axhline(y=100 * thr, color='lightgrey', linewidth=0.25)
+            ax[m].axvline(x=varexp[thr], color='C{}'.format(i),
+                          linewidth=2, linestyle=':')
+            ax[m].text(0, 100 * thr, '{:.0f}'.format(100 * thr),
+                       fontsize='x-small', bbox=bbox_txt)
+            ax[m].text(varexp[thr][0], 25,
+                       '{} components explain\n{:.0f}% of variance'.format(
+                            varexp[thr][0], 100 * thr),
+                       rotation=90,
+                       horizontalalignment='center',
+                       fontsize='xx-small',
+                       bbox=bbox_txt)
+
+        ax[m].set_yticks([])
+        ax[m].set_yticklabels([])
+        for tick in ax[m].xaxis.get_major_ticks():
+            tick.label.set_fontsize('x-small')
+            tick.label.set_rotation('vertical')
+        for side in ['top', 'right', 'left']:
+            ax[m].spines[side].set_color('none')
+            ax[m].spines[side].set_visible(False)
+
+    if output_file is not None:
+        figure = plt.gcf()
+        figure.savefig(output_file, bbox_inches='tight')
+        plt.close(figure)
+        figure = None
+        return output_file
+    return ax
+
+
+def confounds_correlation_plot(confounds_file, output_file=None, figure=None,
+                               reference='global_signal', max_dim=70):
+    """
+    Parameters
+    ----------
+    confounds_file: str
+        File containing all confound regressors to be included in the
+        correlation plot.
+    output_file: str or None
+        Path where the output figure should be saved. If this is not defined,
+        then the plotting axes will be returned instead of the saved figure
+        path.
+    figure: figure or None
+        Existing figure on which to plot.
+    reference: str
+        `confounds_correlation_plot` prepares a bar plot of the correlations
+        of each confound regressor with a reference column. By default, this
+        is the global signal (so that collinearities with the global signal
+        can readily be assessed).
+    max_dim: int
+        The maximum number of regressors to be included in the output plot.
+        Reductions (e.g., CompCor) of high-dimensional data can yield so many
+        regressors that the correlation structure becomes obfuscated. This
+        criterion selects the `max_dim` regressors that have the largest
+        correlation magnitude with `reference` for inclusion in the plot.
+
+    Returns
+    -------
+    axes and gridspec
+        Plotting axes and gridspec. Returned only if `output_file` is None.
+    output_file: str
+        The file where the figure is saved.
+    """
+    confounds_data = pd.read_table(confounds_file)
+    confounds_data = confounds_data.loc[:, np.logical_not(
+        np.isclose(confounds_data.var(skipna=True), 0))]
+    corr = confounds_data.corr()
+
+    gscorr = corr.copy()
+    gscorr['index'] = gscorr.index
+    gscorr[reference] = np.abs(gscorr[reference])
+    gs_descending = gscorr.sort_values(by=reference,
+                                       ascending=False)['index']
+    n_vars = corr.shape[0]
+    max_dim = min(n_vars, max_dim)
+
+    gs_descending = gs_descending[:max_dim]
+    features = [p for p in corr.columns if p in gs_descending]
+    corr = corr.loc[features, features]
+    np.fill_diagonal(corr.values, 0)
+
+    if figure is None:
+        plt.figure(figsize=(15, 5))
+    gs = mgs.GridSpec(1, 21)
+    ax0 = plt.subplot(gs[0, :10])
+    ax1 = plt.subplot(gs[0, 11:])
+
+    mask = np.zeros_like(corr, dtype=np.bool)
+    mask[np.triu_indices_from(mask)] = True
+    sns.heatmap(corr,
+                linewidths=0.5,
+                cmap='coolwarm',
+                center=0,
+                square=True,
+                ax=ax0)
+    ax0.tick_params(axis='both', which='both', width=0)
+
+    for tick in ax0.xaxis.get_major_ticks():
+        tick.label.set_fontsize('small')
+    for tick in ax0.yaxis.get_major_ticks():
+        tick.label.set_fontsize('small')
+    sns.barplot(data=gscorr,
+                x='index',
+                y=reference,
+                ax=ax1,
+                order=gs_descending,
+                palette='Reds_d',
+                saturation=.5)
+
+    ax1.set_xlabel('Confound time series')
+    ax1.set_ylabel('Magnitude of correlation with {}'.format(reference))
+    ax1.tick_params(axis='x', which='both', width=0)
+    ax1.tick_params(axis='y', which='both', width=5, length=5)
+
+    for tick in ax1.xaxis.get_major_ticks():
+        tick.label.set_fontsize('small')
+        tick.label.set_rotation('vertical')
+    for tick in ax1.yaxis.get_major_ticks():
+        tick.label.set_fontsize('small')
+    for side in ['top', 'right', 'left']:
+        ax1.spines[side].set_color('none')
+        ax1.spines[side].set_visible(False)
+
+    if output_file is not None:
+        figure = plt.gcf()
+        figure.savefig(output_file, bbox_inches='tight')
+        plt.close(figure)
+        figure = None
+        return output_file
+    return [ax0, ax1], gs
